#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "gnome-presence-applet.h"

#include <bonobo/bonobo-ui-component.h>
#include <panel-applet-gconf.h>
#include <libgnomeui/gnome-about.h>

#include <string.h>

#include "prefs-dialog.h"
#include "utils.h"

static void
switch_person(PresenceApplet *applet, GalagoPerson *person)
{
	galago_gtk_presence_icon_set_person(
		GALAGO_GTK_PRESENCE_ICON(applet->icon), person);
	galago_gtk_presence_label_set_person(
		GALAGO_GTK_PRESENCE_LABEL(applet->label), person);

	applet->person = person;

	if (person == NULL)
	{
		char *key;

		key = panel_applet_gconf_get_full_key(applet->panel_applet,
			PRESENCE_MONITOR_KEY_ACTIVE_PERSON);

		gconf_client_unset(applet->gconf_client, key, NULL);

		g_free(key);
	}
	else
	{
		panel_applet_gconf_set_string(applet->panel_applet,
									  PRESENCE_MONITOR_KEY_ACTIVE_PERSON,
									  galago_person_get_id(person), NULL);
	}
}

static void
menu_item_activate_cb(GalagoGtkPresenceMenuItem *menu_item,
					  PresenceApplet *applet)
{
	switch_person(applet, galago_gtk_presence_menu_item_get_person(menu_item));
}

static gboolean
populate_menu(GtkMenu *menu, PresenceApplet *applet)
{
	GSList *slist, *sl;
	GalagoPerson *person;
	gboolean result = FALSE;

	slist = panel_applet_gconf_get_list(applet->panel_applet,
										PRESENCE_MONITOR_KEY_PEOPLE_MONITORED,
										GCONF_VALUE_STRING, NULL);

	for (sl = slist; sl != NULL; sl = sl->next)
	{
		char *uid = (char *)sl->data;
		GtkWidget *menu_item;

		person = galago_core_get_person(uid, FALSE, TRUE);

		g_free(uid);

		if (person == NULL)
			continue;

		result = TRUE;

		menu_item = galago_gtk_presence_menu_item_new(NULL);
		galago_gtk_presence_menu_item_set_person(
			GALAGO_GTK_PRESENCE_MENU_ITEM(menu_item), person);

		galago_gtk_presence_menu_item_set_dim_idle(
			GALAGO_GTK_PRESENCE_MENU_ITEM(menu_item),
			panel_applet_gconf_get_bool(applet->panel_applet,
										PRESENCE_MONITOR_KEY_DIM_IDLE, NULL));
		galago_gtk_presence_menu_item_set_show_idle_times(
			GALAGO_GTK_PRESENCE_MENU_ITEM(menu_item),
			panel_applet_gconf_get_bool(applet->panel_applet,
										PRESENCE_MONITOR_KEY_SHOW_IDLE_TIME,
										NULL));

		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);

		g_signal_connect(G_OBJECT(menu_item), "activate",
						 G_CALLBACK(menu_item_activate_cb), applet);

		gtk_widget_show(menu_item);
	}

	g_slist_free(slist);

	return result;
}

static void
menu_position_cb(GtkMenu *menu, int *x, int *y, gboolean *push_in,
				 gpointer user_data)
{
	PresenceApplet *applet = (PresenceApplet *)user_data;
	GtkWidget *widget = GTK_WIDGET(applet->panel_applet);
	GtkRequisition req;
	gint menu_xpos, menu_ypos;

	gtk_widget_size_request(GTK_WIDGET(menu), &req);
	gdk_window_get_origin(widget->window, &menu_xpos, &menu_ypos);

	menu_xpos += widget->allocation.x;
	menu_ypos += widget->allocation.y;

	if (menu_ypos > gdk_screen_get_height(gtk_widget_get_screen(widget)) / 2)
		menu_ypos -= req.height;
	else
		menu_ypos += widget->allocation.height;

	*x = menu_xpos;
	*y = menu_ypos;

	*push_in = TRUE;
}

static gboolean
popup_menu_cb(GtkWidget *widget, GdkEventButton *event, PresenceApplet *applet)
{
	if (event->type != GDK_BUTTON_PRESS || event->button != 1)
		return FALSE;

	if (applet->menu != NULL)
	{
		gtk_menu_popup(GTK_MENU(applet->menu), NULL, NULL,
					   menu_position_cb, applet,
					   event->button, event->time);
	}

	return TRUE;
}

static void
update_people_list(PresenceApplet *applet)
{
	GSList *slist, *sl;

	if (applet->people_table != NULL)
		g_hash_table_destroy(applet->people_table);

	applet->people_table = g_hash_table_new_full(g_str_hash, g_str_equal,
												 g_free, NULL);

	slist = panel_applet_gconf_get_list(applet->panel_applet,
										PRESENCE_MONITOR_KEY_PEOPLE_MONITORED,
										GCONF_VALUE_STRING, NULL);

	for (sl = slist; sl != NULL; sl = sl->next)
	{
		char *str = (char *)sl->data;

		g_hash_table_insert(applet->people_table, str, GINT_TO_POINTER(1));
	}

	g_slist_free(slist);
}

static void
rebuild_menu(PresenceApplet *applet)
{
	if (applet->menu != NULL)
	{
		gtk_widget_destroy(applet->menu);
		applet->menu = NULL;
	}

	applet->menu = gtk_menu_new();

	if (!populate_menu(GTK_MENU(applet->menu), applet))
	{
		gtk_widget_destroy(applet->menu);
		applet->menu = NULL;
	}
}

static void
set_default_person(PresenceApplet *applet)
{
	char *uid;
	GalagoPerson *person;

	uid = panel_applet_gconf_get_string(applet->panel_applet,
										PRESENCE_MONITOR_KEY_ACTIVE_PERSON,
										NULL);

	if (uid == NULL)
		return;

	person = galago_core_get_person(uid, FALSE, TRUE);

	g_free(uid);

	if (person != NULL)
		switch_person(applet, person);
}

static void
prefs_cb(BonoboUIComponent *uic, gpointer user_data, const gchar *verbname)
{
	PresenceApplet *applet = (PresenceApplet *)user_data;

	if (applet->prefs_dialog != NULL)
		gtk_window_present(GTK_WINDOW(applet->prefs_dialog->dialog));
	else
	{
		applet->prefs_dialog = prefs_dialog_new(applet);

		g_return_if_fail(applet->prefs_dialog != NULL);

		gtk_widget_show(applet->prefs_dialog->dialog);
	}
}

static void
about_cb(BonoboUIComponent *uic, gpointer user_data, const gchar *verbname)
{
	static GtkWidget *dialog = NULL;
	const char *authors[] =
	{
		"Christian Hammond <chipx86@gnupdate.org>", NULL
	};

	if (dialog != NULL)
		gtk_window_present(GTK_WINDOW(dialog));
	else
	{
		dialog = gnome_about_new(_("Presence Monitor Applet"), PACKAGE_VERSION,
			"Copyright (C) 2004-2005 Christian Hammond",
			_("An applet to help keep watch on the presence of other people."),
			authors, NULL, NULL, NULL);

		g_signal_connect(G_OBJECT(dialog), "destroy",
						 G_CALLBACK(gtk_widget_destroyed), &dialog);

		g_object_add_weak_pointer(G_OBJECT(dialog), (gpointer)&dialog);
		gtk_widget_show(dialog);
	}
}

static void
bonobo_destroy_cb(BonoboObject *object, PresenceApplet *applet)
{
	gconf_client_notify_remove(applet->gconf_client, applet->show_idle_id);
	gconf_client_notify_remove(applet->gconf_client, applet->dim_idle_id);
	gconf_client_notify_remove(applet->gconf_client,
							   applet->people_monitored_id);

	galago_signals_disconnect_by_id(applet->galago_registered_id);
	galago_signals_disconnect_by_id(applet->person_added_id);

	if (applet->people_table != NULL)
	{
		g_hash_table_destroy(applet->people_table);
		applet->people_table = NULL;
	}

	if (applet->menu != NULL)
	{
		gtk_widget_destroy(applet->menu);
		applet->menu = NULL;
	}

	galago_uninit();

	g_free(applet);
}

static const BonoboUIVerb lookup_applet_menu_verbs[] =
{
	BONOBO_UI_VERB("prefs", prefs_cb),
	BONOBO_UI_VERB("about", about_cb),
	BONOBO_UI_VERB_END
};

static void
dim_idle_changed_cb(GConfClient *client, guint cnxn_id, GConfEntry *entry,
					gpointer user_data)
{
	PresenceApplet *applet = (PresenceApplet *)user_data;

	galago_gtk_presence_label_set_dim_idle(
		GALAGO_GTK_PRESENCE_LABEL(applet->label),
		gconf_value_get_bool(gconf_entry_get_value(entry)));

	rebuild_menu(applet);
}

static void
show_idle_time_changed_cb(GConfClient *client, guint cnxn_id,
						  GConfEntry *entry, gpointer user_data)
{
	PresenceApplet *applet = (PresenceApplet *)user_data;

	galago_gtk_presence_label_set_show_idle_times(
		GALAGO_GTK_PRESENCE_LABEL(applet->label),
		gconf_value_get_bool(gconf_entry_get_value(entry)));

	rebuild_menu(applet);
}

static void
people_monitored_changed_cb(GConfClient *client, guint cnxn_id,
							GConfEntry *entry, gpointer user_data)
{
	PresenceApplet *applet = (PresenceApplet *)user_data;

	update_people_list(applet);

	rebuild_menu(applet);
}

/* I hate all of this. PPOs will make this better, if they happen. */
static void
person_added_cb(GalagoService *service, const char *signal,
				GalagoPerson *person, PresenceApplet *applet)
{
	if (applet->people_table == NULL)
		return;

	if (g_hash_table_lookup(applet->people_table,
							galago_person_get_id(person)) != NULL)
	{
		rebuild_menu(applet);
	}
}

static void
galago_registered_cb(void *ptr, const char *signal, PresenceApplet *applet)
{
	rebuild_menu(applet);
	set_default_person(applet);
}

static gboolean
presence_applet_new(PanelApplet *panel_applet)
{
	PresenceApplet *applet;
	GtkWidget *hbox;
	char *key;

	if (!galago_glib_init("gnome-presence-applet", FALSE, NULL))
	{
		GtkWidget *window = gtk_widget_get_toplevel(GTK_WIDGET(panel_applet));
		GtkWidget *dialog = gtk_message_dialog_new(
			GTK_WINDOW(window),
			GTK_DIALOG_MODAL,
			GTK_MESSAGE_ERROR,
			GTK_BUTTONS_CLOSE,
			_("Unable to initialize desktop presence service."));

		gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog),
			_("The GNOME Presence Monitor applet was unable to "
			  "initialize Galago, the desktop presence service. Please "
			  "be sure you have D-BUS and Galago set up properly.\n"));
		gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
		gtk_window_set_title(GTK_WINDOW(dialog),
							 _("GNOME Presence Monitor Error"));
		gtk_window_set_screen(GTK_WINDOW(dialog),
							  gtk_window_get_screen(GTK_WINDOW(window)));

		g_signal_connect_swapped(G_OBJECT(dialog), "response",
								 G_CALLBACK(gtk_widget_destroy), dialog);

		gtk_widget_show(dialog);

		return FALSE;
	}

	panel_applet_set_flags(panel_applet, PANEL_APPLET_EXPAND_MINOR);


	applet = g_new0(PresenceApplet, 1);
	applet->panel_applet = panel_applet;
	applet->panel_size   = panel_applet_get_size(panel_applet);
	applet->orient       = panel_applet_get_orient(panel_applet);

	/* Setup GConf */
	panel_applet_add_preferences(panel_applet,
								 "/schemas/apps/gnome-presence-applet/prefs",
								 NULL);

	applet->gconf_client = gconf_client_get_default();

	/* Create the main hbox */
	hbox = gtk_hbox_new(FALSE, 3);
	gtk_container_add(GTK_CONTAINER(panel_applet), hbox);
	gtk_widget_show(hbox);

	/* Create the icon */
	applet->icon = galago_gtk_presence_icon_new();
	galago_gtk_presence_icon_set_size(GALAGO_GTK_PRESENCE_ICON(applet->icon),
									  GTK_ICON_SIZE_MENU);
	galago_gtk_presence_icon_set_precedence(
		GALAGO_GTK_PRESENCE_ICON(applet->icon), GALAGO_GTK_ICON_PHOTO);
	gtk_box_pack_start(GTK_BOX(hbox), applet->icon, FALSE, FALSE, 0);
	gtk_widget_show(applet->icon);

	/* Create the label */
	applet->label = galago_gtk_presence_label_new();
	galago_gtk_presence_label_set_dim_idle(
		GALAGO_GTK_PRESENCE_LABEL(applet->label),
		panel_applet_gconf_get_bool(applet->panel_applet,
									PRESENCE_MONITOR_KEY_DIM_IDLE, NULL));
	galago_gtk_presence_label_set_show_idle_times(
		GALAGO_GTK_PRESENCE_LABEL(applet->label),
		panel_applet_gconf_get_bool(applet->panel_applet,
									PRESENCE_MONITOR_KEY_SHOW_IDLE_TIME,
									NULL));

	key = panel_applet_gconf_get_full_key(applet->panel_applet,
										  PRESENCE_MONITOR_KEY_DIM_IDLE);
	applet->dim_idle_id =
		gconf_client_notify_add(applet->gconf_client, key,
								dim_idle_changed_cb, applet, NULL, NULL);
	g_free(key);

	key = panel_applet_gconf_get_full_key(applet->panel_applet,
										  PRESENCE_MONITOR_KEY_SHOW_IDLE_TIME);
	applet->show_idle_id =
		gconf_client_notify_add(applet->gconf_client, key,
								show_idle_time_changed_cb, applet, NULL, NULL);
	g_free(key);

	gtk_box_pack_start(GTK_BOX(hbox), applet->label, TRUE, TRUE, 0);
	gtk_widget_show(applet->label);

	g_signal_connect(GTK_OBJECT(applet->panel_applet), "button_press_event",
					 G_CALLBACK(popup_menu_cb), applet);

	gtk_widget_show(GTK_WIDGET(applet->panel_applet));

	/* Load in our XML file for the applet menu. */
	panel_applet_setup_menu_from_file(applet->panel_applet,
									  NULL,
									  PKGDATADIR"/GNOME_PresenceApplet.xml",
									  NULL,
									  lookup_applet_menu_verbs,
									  applet);

	/* Setup signals for our applet. */
	g_signal_connect(panel_applet_get_control(applet->panel_applet), "destroy",
					 G_CALLBACK(bonobo_destroy_cb), applet);

	update_people_list(applet);

	/* Setup our menu. */
	rebuild_menu(applet);

	/* Set up our default displayed person. */
	set_default_person(applet);

	/* Setup our people monitored gconf notification. */
	key = panel_applet_gconf_get_full_key(applet->panel_applet,
		PRESENCE_MONITOR_KEY_PEOPLE_MONITORED);
	applet->people_monitored_id =
		gconf_client_notify_add(applet->gconf_client, key,
								people_monitored_changed_cb, applet, NULL,
								NULL);
	g_free(key);

	/* Listen for register signals. */
	applet->galago_registered_id =
		galago_signal_connect(galago_get_core(), "registered",
							  GALAGO_CALLBACK(galago_registered_cb), applet);

	/* Listen for person-added signals. */
	applet->person_added_id =
		galago_signal_connect(galago_get_core(), "person-added",
							  GALAGO_CALLBACK(person_added_cb), applet);

	return TRUE;
}

static gboolean
presence_applet_factory(PanelApplet *applet, const gchar *iid, gpointer data)
{
	if (strcmp(iid, "OAFIID:GNOME_PresenceApplet"))
		return FALSE;

	return presence_applet_new(applet);
}

PANEL_APPLET_BONOBO_FACTORY("OAFIID:GNOME_PresenceApplet_Factory",
							PANEL_TYPE_APPLET,
							"gnome-presence-applet",
							"0",
							presence_applet_factory,
							NULL);
