#include "utils.h"
#include <glib.h>
#include <string.h>

char *
serialize_account(const GalagoAccount *account)
{
	GalagoService *service;

	g_return_val_if_fail(account != NULL, NULL);

	service = galago_account_get_service(account);

	return g_strdup_printf("%s|%s",
						   galago_service_get_id(service),
						   galago_account_get_username(account));
}

GalagoAccount *
unserialize_account(const char *str, char **ret_account_name,
					char **ret_service_id)
{
	char *temp, *c, *account_name, *service_id;
	GalagoService *service;
	GalagoAccount *account;

	g_return_val_if_fail(str != NULL || *str != '\0', NULL);

	temp = g_strdup(str);

	service_id = temp;

	if ((c = strchr(temp, '|')) == NULL)
	{
		g_free(temp);

		return NULL;
	}

	*c = '\0';

	account_name = c + 1;

	service = galago_core_get_service(service_id, FALSE, TRUE);

	if (service == NULL)
	{
		g_free(temp);

		return NULL;
	}

	account = galago_service_get_account(service, account_name, TRUE);

	if (ret_account_name != NULL)
		*ret_account_name = g_strdup(account_name);

	if (ret_service_id != NULL)
		*ret_service_id = g_strdup(service_id);

	g_free(temp);

	return account;
}

